#include "config.h"

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "test-stuff.h"


static int return_value = TEST_SUCCESS;

void
success(int test_num, char *test_title, char *format, ...)
{
    va_list ap;

    va_start(ap, format);

    printf("SUCCESS %d: %s : ", test_num, test_title);
    vprintf(format, ap);
    printf("\n");
    fflush(stdout);

    va_end(ap);
}

void
failure(int test_num, char *test_title, char *format, ...)
{
    va_list ap;

    va_start(ap, format);

    return_value = TEST_FAILURE;
    
    printf("FAILURE %d: %s : ", test_num, test_title);
    vprintf(format, ap);
    printf("\n");
    fflush(stdout);
    
    va_end(ap);
}

int
get_rv(void)
{
    return return_value;
}

static int
files_return(int ret, const char* msg)
{
    printf(msg);
    return ret;
}

int
files_compare(const gchar* f1, const gchar* f2)
{
    gchar buf1[512];
    gchar buf2[512];
    int fd1, fd2;
    int amount_read1, amount_read2;
    
    fd1 = open(f1, O_RDONLY);
    fd2 = open(f2, O_RDONLY);

    do 
    {
        int memcmp_ret;
        
        amount_read1 = read(fd1, buf1, 512);
        amount_read2 = read(fd2, buf2, 512);

        if(amount_read1 > amount_read2)
            return files_return(1, "read1 > read2");
        if(amount_read1 < amount_read2)
            return files_return(-1, "read1 < read2");

        if((memcmp_ret = memcmp(buf1, buf2, amount_read1)) != 0)
        {
            return files_return(memcmp_ret, "memcmp return");
        }
        
    } while(amount_read1 == 512);
    
    return 0;
}

void
write_dom_node_to_file(xmlNodePtr node, int fd)
{
    FILE *out;

    out = fdopen(fd, "w");

    xmlElemDump(out, NULL, node);

    fclose(out);
}

gboolean
print_dom_tree(gpointer data_for_children, GSList* data_from_children,
               GSList* sibling_data, gpointer parent_data,
               gpointer global_data, gpointer *result, const gchar *tag)
{
    if (parent_data == NULL) 
    {
        xmlElemDump((FILE*)global_data, NULL, (xmlNodePtr)data_for_children);
        xmlFreeNode(data_for_children);
    }
    return TRUE;
}

gint
get_random_int_in_range(int start, int end)
{
    return start + (int)((double)end * rand() / (RAND_MAX + 1.0));
}

gchar
get_random_character(void)
{
    return 'A' + get_random_int_in_range(1, 25);
}

gchar*
get_random_string(void)
{
    gchar *ret;
    int len;
    int i;
    
    len = get_random_int_in_range(5, 10);

    ret = g_new0(gchar, len);

    for(i = 0; i < len - 1; i++)
    {
        ret[i] = get_random_character();
    }
    return ret;
}

Timespec*
get_random_timespec(void)
{
    Timespec *ret;

    ret = g_new(Timespec, 1);

    ret->tv_sec = rand();
    ret->tv_nsec = rand();

    return ret;
}

gint64
get_random_gint64(void)
{
    gint64 ret = 0;

    ret = rand();
    ret <<= 32;
    ret += rand();
    
    return ret;
}

double
get_random_double(void)
{
    union 
    {
        double d;
        gint64 i;
    } ret;
    

    ret.i = get_random_gint64();
    
    return ret.d;
}

gnc_numeric
get_random_gnc_numeric(void)
{
    return gnc_numeric_create(get_random_gint64(), rand());
}

gnc_commodity*
get_random_gnc_commodity_ref(void)
{
    gchar *name;
    gchar *mnemonic;
    gnc_commodity *ret;
    
    name = get_random_string();
    mnemonic = get_random_string();
    
    ret = gnc_commodity_new("", name, mnemonic, "", 0);

    g_free(name);
    g_free(mnemonic);

    return ret;
}

GUID*
get_random_guid(void)
{
    GUID *ret;
    
    ret = g_new(GUID, 1);
    guid_new(ret);

    return ret;
}

GList*
get_random_glist(void)
{
    GList *ret = NULL;
    int i;
    int count = get_random_int_in_range(1, 5);
    
    for(i = 0; i < count; i++)
    {
        ret = g_list_prepend(ret, (gpointer)get_random_kvp_value(-2));
    }
    
    return ret;
}

bin_data*
get_random_binary_data(void)
{
    int len;
    bin_data *ret;

    len = get_random_int_in_range(20,100);
    ret = g_new(bin_data, 1);
    ret->data = g_new(char, len);
    ret->len = len;

    for(len--; len >= 0; len--)
    {
        ret->data[len] = (char)get_random_int_in_range(0,255);
    }

    return ret;
}

kvp_frame*
get_random_kvp_frame(void)
{
    kvp_frame *ret;
    int vals_to_add;
    
    ret = kvp_frame_new();

    vals_to_add = get_random_int_in_range(1,10);

    for(;vals_to_add > 0; vals_to_add--)
    {
        gchar *key;
        kvp_value *val;
        
        key = get_random_string();
        val = get_random_kvp_value(-1);

        kvp_frame_set_slot_nc(ret, key, val);

        g_free(key);
    }
    
    return ret;
}

kvp_value*
get_random_kvp_value(int type)
{
    int datype = type;

    if(datype == -1)
    {
        datype = get_random_int_in_range(KVP_TYPE_GINT64, KVP_TYPE_FRAME + 1);
    }

    if(datype == -2)
    {
        datype = get_random_int_in_range(KVP_TYPE_GINT64, KVP_TYPE_FRAME);
    }
    
    switch(datype)
    {
    case KVP_TYPE_GINT64:
        return kvp_value_new_gint64(get_random_gint64());
        break;

    case KVP_TYPE_DOUBLE:
        return kvp_value_new_double(get_random_double());
        break;

    case KVP_TYPE_NUMERIC:
        return kvp_value_new_gnc_numeric(get_random_gnc_numeric());
        break;

    case KVP_TYPE_STRING:
    {
        gchar *tmp_str;
        kvp_value *ret;
        tmp_str = get_random_string();
        ret = kvp_value_new_string(tmp_str);
        g_free(tmp_str);
        return ret;
    }
        break;

    case KVP_TYPE_GUID:
    {
        GUID *tmp_guid;
        kvp_value *ret;
        tmp_guid = get_random_guid();
        ret = kvp_value_new_guid(tmp_guid);
        g_free(tmp_guid);
        return ret;
    }
        break;

    case KVP_TYPE_BINARY:
    {
        bin_data *tmp_data;
        kvp_value *ret;
        tmp_data = get_random_binary_data();
        ret = kvp_value_new_binary(tmp_data->data, tmp_data->len);
        g_free(tmp_data->data);
        g_free(tmp_data);
        return ret;
    }
        break;
 
    case KVP_TYPE_GLIST:
        return kvp_value_new_glist_nc(get_random_glist());
        break;

    case KVP_TYPE_FRAME:
    {
        kvp_frame *tmp_frame;
        kvp_value *ret;
        tmp_frame = get_random_kvp_frame();
        ret = kvp_value_new_frame(tmp_frame);
        kvp_frame_delete(tmp_frame);
        return ret;
    }
        break;

    default:
        return NULL;
        break;
    }
}
